import connexion
import six

from swagger_server.models.people import People  # noqa: E501
from swagger_server.models.people_crud import PeopleCRUD  # noqa: E501
from swagger_server.models.peoples import Peoples  # noqa: E501
from swagger_server.models.departments import Departments  # noqa: E501
from swagger_server import util

from . import check_error
import json


@check_error
def add_people(body=None):  # noqa: E501
    """Add a new people

    create new people # noqa: E501

    :param body: People object that needs to be added
    :type body: dict | bytes

    :rtype: People
    """
    if connexion.request.is_json:
        body = PeopleCRUD.from_dict(connexion.request.get_json())  # noqa: E501
    return Peoples().append(body)

@check_error
def delete_people(peopleId):  # noqa: E501
    """Deletes a people

    delete people by Id # noqa: E501

    :param peopleId: People id to delete
    :type peopleId: int

    :rtype: None
    """
    people = Peoples().getPeopleById(peopleId)
    Peoples().remove(people)
    return json.dumps({'status' : 'ok'}), 200

@check_error
def get_people(peopleId):  # noqa: E501
    """Find people by ID

    Returns a people # noqa: E501

    :param peopleId: ID of people that needs to be fetched
    :type peopleId: int

    :rtype: People
    """
    return Peoples().getPeopleById(peopleId)

@check_error
def get_peoples(gender=None, department=None):  # noqa: E501
    """list of people

    Returns list of people # noqa: E501

    :param gender: filter by gender
    :type gender: str
    :param department: filter by department
    :type department: int

    :rtype: List[People]
    """
    return [x for x in [y for y in Peoples() if y.department.id == department or department==None ] if x.gender == gender or gender==None]

def update_people(peopleId, body):  # noqa: E501
    """Updates a people with form data

    update people by Id # noqa: E501

    :param peopleId: ID of people that needs to be updated
    :type peopleId: int
    :param body: People object that needs to be updated
    :type body: dict | bytes

    :rtype: None
    """
    if connexion.request.is_json:
        body = People.from_dict({**connexion.request.get_json(), 
                                 "id":peopleId,
                                 "department":Departments().getDepartmentById(connexion.request.get_json()["department"]).to_dict()})
    return Peoples().update(body)
