# coding: utf-8
import uuid

from .departments import Departments
from .people import People
from .people_crud import PeopleCRUD
from .error_model import Error
from . import Singleton

import logging

class DuplicateError(Error):
    """
    Error - Duplicate People
    """ 
    def __init__(self, key=""):
        Error.__init__(self,
                       status=400,
                       title="Duplicate People",
                       type="RG-011",
                       key=key)

    def __str__(self):
        return "%s already exist" % self._key

class NotPeopleError(Error):
    """
    Error - People Error
    """
    def __init__(self):
        Error.__init__(self,
                       status=405,
                       title="People error",
                       type="INPUT-013")

    def __str__(self):
        return "It is not People or PeopleCRUD object"

class NotFoundPeopleError(Error):
    """
    Error - Not Found People
    """
    def __init__(self, key=""):
        Error.__init__(self,
                       status=404,
                       title="Not Found People",
                       type="RG-012",
                       key=key)

    def __str__(self):
        return "People %s not found" % self._key

class Peoples(list, metaclass=Singleton):
    """
    Peoples - manage list of People
    """
    def __init__(self):
        list.__init__(self)
    
    def _get_id(self):
        return uuid.uuid1().int 

    def append(self, el):
        if type(el) is PeopleCRUD:
            el = People.from_dict({"id":self._get_id(),
                                   **el.to_dict(),
                                   "department": Departments().getDepartmentById(el.department).to_dict()})
        if type(el) is People:
            if el in self:
                raise DuplicateError(el)
            list.append(self,el)
            logging.getLogger('connexion.app').warning("create people %s" % el.id)
            return el
        raise NotPeopleError()
    
    def getPeopleById(self, id):
        for people in self:
            if people.id == id:
                return people
        raise NotFoundPeopleError(id)

    def update(self, el):
        if type(el) is not People:
            raise NotPeopleError()
        self.remove(self.getPeopleById(el.id))
        self.append(el)
        logging.getLogger('connexion.app').warning("update people %s" % el.id)
        return el
    
    def remove(self, el):
        list.remove(self,el)
        logging.getLogger('connexion.app').warning("remove people %s" % el.id)


# For manage relation between People and list of Departments
from swagger_server.models.departments import Departments  # noqa: E501
from swagger_server.models.department import Department  # noqa: E501

def get_department(cls):
    return cls._department

def set_department(cls, department):
    cls._department = Departments().getDepartmentById(department.id)

People.department = property(get_department,set_department)

def get_cntpeople(cls):
    return len([people for people in Peoples() if people.department==cls])

def set_cntpeople(cls, department):
    pass

Department.cntpeople = property(get_cntpeople,set_cntpeople)